package com.javacodegeeks.akka.actor;

import static akka.pattern.PatternsCS.ask;

import java.io.IOException;
import java.time.Duration;
import java.util.Collection;
import java.util.Optional;
import java.util.concurrent.CompletableFuture;

import akka.actor.ActorRef;
import akka.actor.ActorSystem;
import akka.actor.Props;

@SuppressWarnings("unchecked")
public class Starter {
    public static void main(String[] args) throws IOException {
        final ActorSystem system = ActorSystem.create("library-system");
        final LibraryService libraryService = new LibraryServiceImpl();
        final ActorRef library = system.actorOf(Props.create(LibraryActor.class, libraryService), "library");

        final CompletableFuture<Collection<Book>> books =  ask(library, new GetBooks(), Duration.ofMillis(1000))
            .thenApply(e -> (Collection<Book>)e)
            .toCompletableFuture();
        System.out.println(books.join());
        
        final CompletableFuture<Book> add =  ask(library, new AddBook("111", "Just a Book"), Duration.ofMillis(1000))
                .thenApply(e -> (Book)e)
                .toCompletableFuture();
        System.out.println(add.join());
        
        final CompletableFuture<Optional<Book>> book =  ask(library, new FindBook("111"), Duration.ofMillis(1000))
                .thenApply(e -> (Optional<Book>)e)
                .toCompletableFuture();
        System.out.println(book.join());
        
        system.terminate();
    }
}
