package com.javacodegeeks.akka.http;

import java.io.IOException;
import java.util.concurrent.CompletionStage;

import akka.actor.ActorSystem;
import akka.http.javadsl.ConnectHttp;
import akka.http.javadsl.Http;
import akka.http.javadsl.ServerBinding;
import akka.stream.ActorMaterializer;

public class Starter {
    public static void main(String[] args) throws IOException {
        final ActorSystem system = ActorSystem.create("library-server");
        final Http http = Http.get(system);
        final ActorMaterializer materializer = ActorMaterializer.create(system);
        
        final LibraryRoutes routes = new LibraryRoutes(new LibraryServiceImpl());
        final CompletionStage<ServerBinding> binding = http.bindAndHandle(
                routes.routes().flow(system, materializer),
                ConnectHttp.toHost("localhost", 8686), 
                materializer
            );

        System.out.println("HTTP server is up and running: http://localhost:8686/\nPress RETURN to exit...");
        System.in.read();

        binding
                .thenCompose(ServerBinding::unbind)
                .thenAccept(unbound -> system.terminate());
    }
}
