package com.javacodegeeks.grpc;

import java.io.IOException;

import com.google.protobuf.Empty;
import com.javacodegeeks.library.AddBookRequest;
import com.javacodegeeks.library.Book;
import com.javacodegeeks.library.BookList;
import com.javacodegeeks.library.Filter;
import com.javacodegeeks.library.LibraryGrpc;
import com.javacodegeeks.library.RemoveBookRequest;
import com.javacodegeeks.library.UpdateBookRequest;

import io.grpc.Server;
import io.grpc.ServerBuilder;
import io.grpc.stub.StreamObserver;

public class Starter {
    public static void main(String[] args) throws IOException, InterruptedException {
      final Server server = ServerBuilder.forPort(51051)
          .addService(new LibraryImpl(new LibraryServiceImpl()))
          .build();

      Runtime.getRuntime().addShutdownHook(new Thread() {
          @Override
          public void run() {
              server.shutdown();
          }
      });

      server.start();
      server.awaitTermination();
    }

    static class LibraryImpl extends LibraryGrpc.LibraryImplBase {
        private final LibraryService libraryService;
        
        public LibraryImpl(final LibraryService libraryService) {
            this.libraryService = libraryService;
        }
        
        @Override
        public void addBook(AddBookRequest request, StreamObserver<Book> responseObserver) {
            final Book book = Book.newBuilder()
                .setIsbn(request.getIsbn())
                .setTitle(request.getTitle())
                .setYear(request.getYear())
                .build();

            responseObserver.onNext(libraryService.addBook(book));
            responseObserver.onCompleted();
        }
        
        @Override
        public void getBooks(Filter request, StreamObserver<BookList> responseObserver) {
            final BookList bookList = BookList
                .newBuilder()
                .addAllBooks(libraryService.getBooks())
                .build();
            responseObserver.onNext(bookList);
            responseObserver.onCompleted();
        }
        
        @Override
        public void updateBook(UpdateBookRequest request, StreamObserver<Book> responseObserver) {
            responseObserver.onNext(libraryService.updateBook(request.getIsbn(), request.getBook()));
            responseObserver.onCompleted();
        }
        
        @Override
        public void removeBook(RemoveBookRequest request, StreamObserver<Empty> responseObserver) {
            libraryService.removeBook(request.getIsbn());
            responseObserver.onCompleted();
        }
    }
}
