package com.javacodegeeks.reactive.grpc;

import java.io.IOException;

import com.google.protobuf.Empty;
import com.javacodegeeks.library.AddBookRequest;
import com.javacodegeeks.library.Book;
import com.javacodegeeks.library.BookList;
import com.javacodegeeks.library.Filter;
import com.javacodegeeks.library.RemoveBookRequest;
import com.javacodegeeks.library.RxLibraryGrpc;
import com.javacodegeeks.library.UpdateBookRequest;

import io.grpc.Server;
import io.grpc.ServerBuilder;
import io.reactivex.Single;

public class Starter {
    public static void main(String[] args) throws IOException, InterruptedException {
      final Server server = ServerBuilder.forPort(51051)
          .addService(new LibraryImpl(new LibraryServiceImpl()))
          .build();

      Runtime.getRuntime().addShutdownHook(new Thread() {
          @Override
          public void run() {
              server.shutdown();
          }
      });

      server.start();
      server.awaitTermination();
    }

    static class LibraryImpl extends RxLibraryGrpc.LibraryImplBase {
        private final LibraryService libraryService;
        
        public LibraryImpl(final LibraryService libraryService) {
            this.libraryService = libraryService;
        }
        
        @Override
        public Single<Book> addBook(Single<AddBookRequest> request) {
            return request
                .map(r -> 
                    Book
                        .newBuilder()
                        .setIsbn(r.getIsbn())
                        .setTitle(r.getTitle())
                        .setYear(r.getYear())
                        .build())
                .map(libraryService::addBook);
        }
        
        @Override
        public Single<BookList> getBooks(Single<Filter> request) {
            return request
                .map(r -> 
                    BookList
                        .newBuilder()
                        .addAllBooks(libraryService.getBooks())
                        .build());
        }
        
        @Override
        public Single<Book> updateBook(Single<UpdateBookRequest> request) {
            return request
                .map(r -> libraryService.updateBook(r.getIsbn(), r.getBook()));
        }
        
        @Override
        public Single<Empty> removeBook(Single<RemoveBookRequest> request) {
            return request
                .map(r -> {
                    libraryService.removeBook(r.getIsbn());
                    return Empty.newBuilder().build();
                });
        }
    }
}
