package com.javacodegeeks.vertx;

import io.vertx.core.Vertx;
import io.vertx.core.http.HttpServer;
import io.vertx.core.json.Json;
import io.vertx.ext.web.Router;
import io.vertx.ext.web.handler.BodyHandler;

public class Starter {
    public static void main(String[] args) {
        final Vertx vertx = Vertx.vertx();
        
        final HttpServer server = vertx.createHttpServer();
        final Router router = Router.router(vertx);
        final LibraryService libraryService = new LibraryServiceImpl();
        
        router
            .get("/library/books")
            .produces("application/json")
            .handler(context ->
                context
                    .response()
                    .putHeader("Content-Type", "application/json")
                    .end(Json.encodePrettily(libraryService.getBooks()))
            );
            
        router
            .get("/library/books/:isbn")
            .produces("application/json")
            .handler(context ->
                libraryService
                    .findBook(context.request().getParam("isbn"))
                    .ifPresentOrElse(
                        book -> context
                                .response()
                                .putHeader("Content-Type", "application/json")
                                .end(Json.encodePrettily(book)),
                        () -> context
                                .response()
                                .setStatusCode(204)
                                .putHeader("Content-Type", "application/json")
                                .end()
                    )
            );
        
        router
            .post("/library/books")
            .consumes("application/json")
            .produces("application/json")
            .handler(BodyHandler.create())
            .handler(context -> {
                final Book book = libraryService
                    .addBook(context.getBodyAsJson().mapTo(Book.class));
                
                context
                    .response()
                    .putHeader("Content-Type", "application/json")
                    .end(Json.encodePrettily(book));
            });
        
        server.requestHandler(router::accept);
        server.listen(4567);
    }
}
